from .models import * 
from rest_framework import serializers
from brvisitApp.models import PatientVisit
from brsessionApp.models import PatientSessionItem
from datetime import date
#from django.db.models import Max

############## Patient%Module #################

class PatientSerializer(serializers.ModelSerializer):
    #new_code_value = serializers.SerializerMethodField()
    last_visit = serializers.DateTimeField(format="%Y-%m-%d",required=False)
    pt_age = serializers.SerializerMethodField()
    #pt_nationality = serializers.CharField(source="nationality.name",default="",read_only=True)
    #pt_complaint = serializers.CharField(source="complaint.name",default="",read_only=True)
    pt_location = serializers.CharField(source="location.name",default="",read_only=True)
    pt_branch = serializers.CharField(source="branch.name",default="",read_only=True)
    pt_foundway = serializers.CharField(source="foundway.name",default="",read_only=True)
    today = date.today()
    #code = serializers.IntegerField(required=False)
    class Meta:
        model = Patient
        fields="__all__"
        #optional_fields = ['code', ]# not working 
        #extra_kwargs = {"code": {"required": False, "allow_null": True}}
        #read_only_fields = ('code',)
    def to_representation(self, instance):
        obj = super().to_representation(instance)
        if obj['next_consult'] : obj['next_consult'] = instance.next_consult.strftime("%Y-%m-%d")
        else : obj['next_consult'] = "غير محدد"
        return obj

    def get_pt_age(self, obj):
        #https://stackoverflow.com/questions/2217488/age-from-birthdate-in-python
        birth_date = obj.birth_date
        return self.today.year - birth_date.year - ((self.today.month, self.today.day) < (birth_date.month, birth_date.day))
    # def get_validation_exclusions(self):
    #     exclusions = super(PatientSerializer, self).get_validation_exclusions()
    #     return exclusions + ['code']
    # def get_new_code_value(self, obj):
    #     max_code = Patient.objects.aggregate(Max('code'))
    #     if max_code["code__max"] : new_code_value = int(max_code["code__max"])+1
    #     else : new_code_value = 1
    #     return new_code_value
    
class PatientInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = PatientInfo
        fields="__all__"

class PatientInfoMiniSerializer(serializers.ModelSerializer):
    fullname = serializers.CharField(source="patient.fullname",default="",read_only=True)
    number_1 = serializers.CharField(source="patient.number_1",default="",read_only=True)
    number_2 = serializers.CharField(source="patient.number_2",default="",read_only=True)
    pt_dr_notes = serializers.CharField(source="patient.dr_notes",default="",read_only=True)
    pt_status_name = serializers.CharField(source="pt_status.name",default="",read_only=True)
    last_visit = serializers.DateTimeField(format="%Y-%m-%d",default="",read_only=True)
    last_session = serializers.DateTimeField(format="%Y-%m-%d",default="",read_only=True)
    created_by__name = serializers.CharField(source="patient.created_by.name",default="",read_only=True)
    marketing_code = serializers.SerializerMethodField()
    class Meta:
        model = PatientInfo
        fields=["patient","fullname","number_1","number_2","pt_status_name","last_visit","last_session","pt_dr_notes","marketing_code","created_by__name"]

    def get_marketing_code(self,obj):
        return obj.get_marketing_code_display()
     
class PatientMiniSerializer(serializers.ModelSerializer):
    class Meta:
        model = Patient
        fields=["id","fullname","code",'number_1','number_2','rfid']

class PtAttachmentSerializer(serializers.ModelSerializer):
    creation_day= serializers.SerializerMethodField()
    class Meta:
        model = PtAttachment
        fields="__all__"
        read_only_fields=("creation_day",)
        extra_kwargs = {"pa_file":{"required":True}}
    
    def to_representation(self, instance):
        res = super().to_representation(instance)
        res['other_attach'] = ExtraPtAttachmentSerializer(ExtraPtAttachment.objects.filter(pt_attachment=instance),many=True).data
        return res

    def get_creation_day(self, obj):
        #return obj.created_at.strftime("%Y-%m-%d")
        #return obj.created_at.strftime("%d/%m/%Y")
        return obj.created_at.strftime("%Y/%m/%d")

# class PtAttachmentSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = PtAttachment
#         fields="__all__"
class PtAttachmentUpdateSerializer(serializers.ModelSerializer):
    patient = serializers.SerializerMethodField(required=False)
    class Meta:
        model = PtAttachment
        fields="__all__"

class ExtraPtAttachmentSerializer(serializers.ModelSerializer):
    class Meta:
        model = ExtraPtAttachment
        fields="__all__"
############## END Patient%Module #################


class PatientVisitMiniSerializer(serializers.ModelSerializer):
    medicalservice_name = serializers.CharField(source="medicalservice.name",default="",read_only=True)
    doctor_name = serializers.CharField(source="doctor.name",default="",read_only=True)   
    next_consult_date = serializers.DateField(format="%Y-%m-%d",required=False)
    recommended_session_name = serializers.CharField(source="recommended_session.name",default="",read_only=True)  
    class Meta:
        model = PatientVisit
        fields="__all__"
        read_only_fields = ("id","medicalservice_name",'notes_for_rcptn','visit_day','doctor_name','recommended_session_name','recommended_session_id',
                            'sessions_min','sessions_max',"next_consult_date",'current_visit_notes_dr','next_visit_notes_dr',"next_consult_date")


class PtEvaluationCardSerializer(serializers.ModelSerializer):
    class Meta:
        model = PtEvaluationCard
        fields="__all__"

class PtEvaluationCardUpdateSerializer(serializers.ModelSerializer):
    patient = serializers.SerializerMethodField(required=False)
    class Meta:
        model = PtEvaluationCard
        fields="__all__"

class PtRomAssessmentSerializer(serializers.ModelSerializer):
    visit_day = serializers.DateField(format="%d-%m-%Y")
    class Meta:
        model = PtRomAssessment
        fields="__all__"

class IrregularPtDataSerializer(serializers.ModelSerializer):
    br_created_by__name = serializers.CharField(source="created_by.name",default="",read_only=True)
    br_created_at__date = serializers.DateField(format="%d-%m-%Y",default="",read_only=True)
    class Meta:
        model = IrregularPtData
        fields="__all__"

    def to_representation(self, instance):
        obj = super().to_representation(instance)
        obj["br_created_by__name"] = instance.br_created_by.name if instance.br_created_by else ""
        obj["br_created_at__date"] = instance.br_created_at.strftime("%Y-%m-%d") if instance.br_created_at else ""
        return obj